<?php

/**
 * Manage comments in an Onpub database.
 *
 * This class contains the methods to manage the data contained in an
 * {@link http://onpubco.com/pdfs/onpub_schema.pdf OnpubComments table}.
 *
 * @author {@link mailto:corey@onpubco.com Corey HM Taylor}
 * @copyright Onpub (TM) Copyright (C) 2007, Onpubco
 * {@link http://onpubco.com/}
 * @license http://www.gnu.org/copyleft/gpl.html GNU General Public License
 * Version 2
 * @package OnpubAPI
 * @version 0.8.6
 */
class OnpubComments
{
    private $pdo;
    /**
     * Controls whether or not the methods in this class use transactions when
     * they access the database.
     *
     * By default, this is set to true and database transactions will be used
     * where necessary. Set this to false if you want to manage transactions
     * yourself.
     * @var bool
     */
    public $enableTransactions;

    /**
     * Connect to an Onpub database.
     *
     * All the methods in this class which query the database use the database
     * connection provided by the PDO object required by this constructor.
     * Currently, Onpub only supports MySQL as a database for storing content.
     * Therefore, when constructing the PDO object, only the
     * {@link PHP_MANUAL#ref.pdo-mysql PDO_MYSQL} driver is supported
     * as a PDO {@link PHP_MANUAL#ref.pdo-mysql.connection data source}.
     *
     * All the methods in this class require the
     * {@link http://onpubco.com/pdfs/onpub_schema.pdf Onpub schema}
     * to be installed in the PDO-connected database. The {@link OnpubDatabase}
     * class contains methods to manage the Onpub schema. The Onpub
     * schema can also be installed by clicking the "Install the schema" link
     * once logged in to the Onpub web interface. The schema generally only has
     * to be installed once per database.
     *
     * @param PDO $pdo A {@link PHP_MANUAL#function.pdo-construct PHP Data Object}.
     * @param bool $enableTransactions If true (the default), the methods in
     * this class that use database transactions will do so. If false, methods
     * that by default use transactions, will not. Only pass false to this
     * argument if you are managing transactions yourself.
     *
     */
    function __construct( PDO $pdo, $enableTransactions = true )
    {
        $this->pdo = $pdo;
        $this->enableTransactions = $enableTransactions;
        $this->pdo->setAttribute( PDO::ATTR_EMULATE_PREPARES, true );
    }

    /**
     * Get the total number of comments in the database.
     *
     * @param int $articleID Get the total number of comments for an article
     *                       identified by its ID.
     * @return int
     */
    public function count( $articleID = null )
    {
        $query = $this->countQuery( $articleID );
        $result = $this->pdo->query( $query );
        OnpubDatabase::verifyQuery( $this->pdo, $result, false );

        if ( !($row = $result->fetch( PDO::FETCH_ASSOC )) ) {
            return 0;
        }

        $result->closeCursor();
        return $row["count"];
    }

    private function countQuery( $articleID = null )
    {
        if ( $articleID ) {
            return "SELECT COUNT(comments.ID) AS count FROM OnpubComments AS comments WHERE articleID = $articleID";
        }
        else {
            return "SELECT COUNT(comments.ID) AS count FROM OnpubComments AS comments";
        }
    }

    /**
     * Delete a comment from the database.
     *
     * @param int $ID ID of the comment to delete.
     * @return int 1 if the comment was deleted, 0 if the comment does not exist in the database.
     */
    public function delete( $ID )
    {
        $oaamaps = new OnpubAAMaps( $this->pdo, false );
        $osamaps = new OnpubSAMaps( $this->pdo, false );

        if ( $this->enableTransactions ) {
            $status = $this->pdo->beginTransaction();
            OnpubDatabase::verifyTransaction( $this->pdo, $status );
        }

        try {
            $oaamaps->delete( $ID, null );
        }
        catch ( PDOException $e ) {
            if ( $this->enableTransactions ) $this->pdo->rollBack();
            throw $e;
        }

        try {
            $osamaps->delete( null, $ID );
        }
        catch ( PDOException $e ) {
            if ( $this->enableTransactions ) $this->pdo->rollBack();
            throw $e;
        }

        $stmt = $this->pdo->prepare( "DELETE FROM OnpubComments WHERE ID = :ID" );
        OnpubDatabase::verifyPrepare( $this->pdo, $stmt, $this->enableTransactions );

        $stmt->bindParam( ':ID', $ID );

        $result = $stmt->execute();
        OnpubDatabase::verifyExecute( $this->pdo, $result, $this->enableTransactions, $stmt->errorInfo() );

        if ( $this->enableTransactions ) {
            $status = $this->pdo->commit();
            OnpubDatabase::verifyTransaction( $this->pdo, $status );
        }

        return $stmt->rowCount();
    }

    /**
     * Search for comments in the database.
     *
     * @param string $keywords Keyword(s) to search for.
     * @return array All the comments which were found as an array of {@link OnpubComment} objects.
     */
    public function search( $keywords, OnpubQueryOptions $queryOptions = null )
    {
        if ( $queryOptions === null ) $queryOptions = new OnpubQueryOptions();

        $query = $this->searchQuery( $keywords, $queryOptions );
        $result = $this->pdo->query( $query );
        OnpubDatabase::verifyQuery( $this->pdo, $result, false );

        $rows = $result->fetchAll( PDO::FETCH_ASSOC );

        $comments = array();
        $content = "";

        if ( $rows ) {
            foreach ( $rows as $row ) {
                if ( $queryOptions->includeContent ) {
                    $content = stripslashes( $row["content"] );
                }
                else {
                    $content = "";
                }

                $comment = new OnpubComment();

                $comment->ID = $row["ID"];
                $comment->imageID = $row["imageID"];
                $comment->articleID = $row["articleID"];
                $comment->authorID = $row["authorID"];
                $comment->parentID = $row["parentID"];
                $comment->title = stripslashes( $row["title"] );
                $comment->content = $content;
                $comment->url = stripslashes( $row["url"] );
                $comment->setCreated( new DateTime( $row["created"] ) );
                $comment->setModified( new DateTime( $row["modified"] ) );

                if ( $row["imageID"] ) {
                    $image = new OnpubImage();

                    $image->ID = $row["imageID"];
                    $image->websiteID = $row["websiteID"];
                    $image->fileName = stripslashes( $row["fileName"] );
                    $image->description = stripslashes( $row["description"] );
                    $image->url = stripslashes( $row["imageURL"] );
                    $image->setCreated( new DateTime( $row["imageCreated"] ) );
                    $image->setModified( new DateTime( $row["imageModified"] ) );

                    $comment->image = $image;
                }

                $comments[] = $comment;
            }
        }

        $result->closeCursor();
        return $comments;
    }

    private function searchQuery( $keywords, OnpubQueryOptions $queryOptions = null )
    {
        if ( $queryOptions === null ) $queryOptions = new OnpubQueryOptions();

        $keywords = OnpubDatabase::utf8Decode( trim( $keywords ) );
        $where = "";

        if ( $keywords ) {
            if ( $queryOptions->fullTextSearch ) {
                $where = "WHERE comments.title LIKE '%$keywords%' OR comments.created LIKE '%$keywords%' OR comments.modified LIKE '%$keywords%' OR comments.ID LIKE '%$keywords%' OR comments.content LIKE '%$keywords%' OR comments.url LIKE '%$keywords%'";
            }
            else {
                $where = "WHERE comments.title LIKE '%$keywords%' OR comments.created LIKE '%$keywords%' OR comments.modified LIKE '%$keywords%' OR comments.ID LIKE '%$keywords%' OR comments.url LIKE '%$keywords%'";
            }
        }

        if ( $queryOptions->orderBy ) {
            $orderBy = $queryOptions->orderBy;
        }
        else {
            $orderBy = "";
        }

        if ( $queryOptions->order ) {
            $order = $queryOptions->order;
        }
        else {
            $order = "";
        }

        if ( $queryOptions->includeContent ) {
            $commentColumns = "comments.ID, comments.imageID, comments.articleID, comments.authorID, comments.parentID, comments.title, comments.content, comments.url, comments.created, comments.modified";
        }
        else {
            $commentColumns = "comments.ID, comments.imageID, comments.articleID, comments.authorID, comments.parentID, comments.title, comments.url, comments.created, comments.modified";
        }

        if ( $queryOptions->dateLimit ) {
            $dateLimit = $queryOptions->dateLimit->format( DATE_ISO8601 );

            if ( $keywords ) {
                $where .= " AND comments.created <= '$dateLimit'";
            }
            else {
                $where = "WHERE comments.created <= '$dateLimit'";
            }
        }

        if ( $orderBy ) {
            if ( $order ) {
                return "SELECT $commentColumns, commentimages.websiteID, commentimages.fileName, commentimages.description, commentimages.url AS imageURL, commentimages.created AS imageCreated, commentimages.modified AS imageModified FROM OnpubComments AS comments LEFT JOIN OnpubImages AS commentimages ON comments.imageID = commentimages.ID $where ORDER BY comments.$orderBy $order";
            }
            else {
                return "SELECT $commentColumns, commentimages.websiteID, commentimages.fileName, commentimages.description, commentimages.url AS imageURL, commentimages.created AS imageCreated, commentimages.modified AS imageModified FROM OnpubComments AS comments LEFT JOIN OnpubImages AS commentimages ON comments.imageID = commentimages.ID $where ORDER BY comments.$orderBy";
            }
        }
        else {
            return "SELECT $commentColumns, commentimages.websiteID, commentimages.fileName, commentimages.description, commentimages.url AS imageURL, commentimages.created AS imageCreated, commentimages.modified AS imageModified FROM OnpubComments AS comments LEFT JOIN OnpubImages AS commentimages ON comments.imageID = commentimages.ID $where";
        }
    }

    /**
     * Get a single comment from the database.
     *
     * @param int $ID ID of the comment to get.
     * @param OnpubQueryOptions $queryOptions Optional database query options.
     * @return OnpubComment An {@link OnpubComment} object. NULL if the comment does not exist in the database.
     */
    public function get( $ID, OnpubQueryOptions $queryOptions = null )
    {
        if ( $queryOptions === null ) $queryOptions = new OnpubQueryOptions();

        $query = $this->getQuery( $ID, $queryOptions );
        $result = $this->pdo->query( $query );
        OnpubDatabase::verifyQuery( $this->pdo, $result, false );
        $rows = $result->fetchAll( PDO::FETCH_ASSOC );

        if ( !$rows ) {
            return null;
        }

        $row = $rows[0];

        if ( $queryOptions->includeContent ) {
            $content = stripslashes( $row["content"] );
        }
        else {
            $content = "";
        }

        $comment = new OnpubComment();

        $comment->ID = $row["ID"];
        $comment->imageID = $row["imageID"];
        $comment->title = stripslashes( $row["title"] );
        $comment->content = $content;
        $comment->url = stripslashes( $row["url"] );
        $comment->setCreated( new DateTime( $row["created"] ) );
        $comment->setModified( new DateTime( $row["modified"] ) );

        if ( !$queryOptions->includeAuthors ) {
            return $comment;
        }

        if ( $row["imageID"] ) {
            $image = new OnpubImage();

            $image->ID = $row["imageID"];
            $image->websiteID = $row["websiteID"];
            $image->fileName = stripslashes( $row["fileName"] );
            $image->description = stripslashes( $row["description"] );
            $image->url = stripslashes( $row["imageURL"] );
            $image->setCreated( new DateTime( $row["imageCreated"] ) );
            $image->setModified( new DateTime( $row["imageModified"] ) );

            $comment->image = $image;
        }

        $authorassoc = array();

        foreach ( $rows as $row ) {
            $authorID = $row["authorID"];

            if ( $authorID ) {
                $author = new OnpubAuthor();

                $author->ID = $authorID;
                $author->imageID = $row["authorImageID"];
                $author->givenNames = stripslashes( $row["givenNames"] );
                $author->familyName = stripslashes( $row["familyName"] );
                $author->displayAs = stripslashes( $row["displayAs"] );
                $author->setCreated( new DateTime( $row["authorCreated"] ) );
                $author->setModified( new DateTime( $row["authorModified"] ) );

                if ( isset( $authorassoc["$authorID"] ) ) {
                    $author = $authorassoc["$authorID"];
                }
                else {
                    $authorassoc["$authorID"] = $author;
                }

                if ( $row["authorImageID"] ) {
                    $image = new OnpubImage();

                    $image->ID = $row["authorImageID"];
                    $image->websiteID = $row["authorImageWebsiteID"];
                    $image->fileName = stripslashes( $row["authorImageFileName"] );
                    $image->description = stripslashes( $row["authorImageDescription"] );
                    $image->setCreated( new DateTime( $row["authorImageCreated"] ) );
                    $image->setModified( new DateTime( $row["authorImageModified"] ) );

                    $author->image = $image;
                }

                $author = $comment->author;
                $exists = false;

                foreach ( $author as $a ) {
                    if ( $a->ID == $author->ID ) {
                        $exists = true;
                    }
                }

                if ( $exists == false ) {
                    $author[] = $author;
                    $comment->authors = $author;
                }
            }
        }

        $result->closeCursor();
        return $comment;
    }

    private function getQuery( $ID, OnpubQueryOptions $queryOptions = null )
    {
        if ( $queryOptions === null ) $queryOptions = new OnpubQueryOptions();

        if ( $queryOptions->orderBy ) {
            $orderBy = $queryOptions->orderBy;
            $orderBy = "comments.$orderBy";
        }
        else {
            $orderBy = "aamaps.ID";
        }

        if ( $queryOptions->order ) {
            $order = $queryOptions->order;
        }
        else {
            $order = "";
        }

        $where = "WHERE comments.ID = $ID";

        if ( $queryOptions->dateLimit ) {
            $dateLimit = $queryOptions->dateLimit->format( DATE_ISO8601 );
            $where .= " AND comments.created <= '$dateLimit'";
        }

        if ( $queryOptions->includeContent ) {
            $commentColumns = "comments.ID, comments.imageID, title, content, comments.url, comments.created, comments.modified";
        }
        else {
            $commentColumns = "comments.ID, comments.imageID, title, comments.url, comments.created, comments.modified";
        }

        if ( $queryOptions->includeAuthors ) {
            return "SELECT $commentColumns, commentimages.websiteID, commentimages.fileName, commentimages.description, commentimages.url AS imageURL, commentimages.created AS imageCreated, commentimages.modified AS imageModified, authors.ID AS authorID, authors.imageID AS authorImageID, givenNames, familyName, displayAs, authors.created AS authorCreated, authors.modified AS authorModified, authorimages.websiteID AS authorImageWebsiteID, authorimages.fileName AS authorImageFileName, authorimages.description AS authorImageDescription, authorimages.url AS authorImageURL, authorimages.created AS authorImageCreated, authorimages.modified AS authorImageModified FROM OnpubComments AS comments LEFT JOIN OnpubImages AS commentimages ON comments.imageID = commentimages.ID LEFT JOIN OnpubAAMaps AS aamaps ON comments.ID = aamaps.commentID LEFT JOIN OnpubAuthors AS authors ON aamaps.authorID = authors.ID LEFT JOIN OnpubImages AS authorimages ON authors.imageID = authorimages.ID $where ORDER BY $orderBy $order";
        }
        else {
            return "SELECT $commentColumns FROM OnpubComments as comments WHERE ID = $ID";
        }
    }

    /**
     * Get the ID of a comment in the database.
     *
     * Looks for a comment in the database with the same title and content.
     * If there's a match, the ID of the comment is returned.
     *
     * @param OnpubComment $comment Comment whose ID you want to get.
     * @return int The ID of the comment. NULL if the comment does not exist in the database.
     */
    public function getID( OnpubComment $comment )
    {
        $stmt = $this->pdo->prepare( "SELECT ID FROM OnpubComments WHERE title = :title AND content = :content" );
        OnpubDatabase::verifyPrepare( $this->pdo, $stmt, false );

        $title = OnpubDatabase::utf8Decode( trim( $comment->title ) );
        $content = OnpubDatabase::utf8Decode( trim( $comment->content ) );

        $stmt->bindParam( ':title', $title );
        $stmt->bindParam( ':content', $content );
        $result = $stmt->execute();
        OnpubDatabase::verifyExecute( $this->pdo, $result, false, $stmt->errorInfo() );

        if ( ($row = $stmt->fetch( PDO::FETCH_ASSOC )) ) {
            return $row["ID"];
        }

        return null;
    }

    /**
     * Insert new comment(s) into the database.
     *
     * @param mixed $comments A single {@link OnpubComment} object or an array of {@link OnpubComment} objects (to insert multiple comments at a time).
     * @return mixed The ID(s) of the new comment(s). An int will be returned if a single comment was inserted. An array of ints will be returned if multiple comments were inserted.
     * @throws PDOException if there's a database error.
     */
    public function insert( $comments )
    {
        $oimages = new OnpubImages( $this->pdo, false );
        $oauthors = new OnpubAuthors( $this->pdo, false );
        $IDs = array();
        $isArray = true;

        if ( !is_array( $comments ) ) {
            $comments = array( $comments );
            $isArray = false;
        }

        if ( $this->enableTransactions ) {
            $status = $this->pdo->beginTransaction();
            OnpubDatabase::verifyTransaction( $this->pdo, $status );
        }

        $stmt = $this->pdo->prepare( "INSERT INTO OnpubComments (ID, imageID, articleID, authorID, parentID, title, content, url, created, modified) VALUES (:ID, :imageID, :articleID, :authorID, :parentID, :title, :content, :url, :created, :modified)" );
        OnpubDatabase::verifyPrepare( $this->pdo, $stmt, $this->enableTransactions );

        foreach ( $comments as $comment ) {
            if ( $comment->image ) {
                try {
                    $imageID = $oimages->insert( $comment->image );
                    $comment->imageID = $imageID;
                }
                catch ( PDOException $e ) {
                    if ( $this->enableTransactions ) $this->pdo->rollBack();
                    throw $e;
                }
            }

            if ( $comment->author ) {
                try {
                    $authorID = $oauthors->insert( $comment->author );
                    $comment->authorID = $authorID;
                }
                catch ( PDOException $e ) {
                    if ( $this->enableTransactions ) $this->pdo->rollBack();
                    throw $e;
                }
            }

            try {
                $ID = $this->getID( $comment );
            }
            catch ( PDOException $e ) {
                if ( $this->enableTransactions ) $this->pdo->rollBack();
                throw $e;
            }

            if ( $ID ) {
                $IDs[] = $ID;
                $comment->ID = $ID;
            }
            else {
                $ID = $comment->ID;
                $imageID = $comment->imageID;
                $articleID = $comment->articleID;
                $authorID = $comment->authorID;
                $parentID = $comment->parentID;
                $title = OnpubDatabase::utf8Decode( trim( $comment->title ) );
                $content = OnpubDatabase::utf8Decode( trim( $comment->content ) );
                $url = OnpubDatabase::utf8Decode( trim( $comment->url ) );
                $created = $comment->getCreated()->format( DATE_ISO8601 );
                $modified = $comment->getModified()->format( DATE_ISO8601 );

                $stmt->bindParam( ':ID', $ID );
                $stmt->bindParam( ':imageID', $imageID );
                $stmt->bindParam( ':articleID', $articleID );
                $stmt->bindParam( ':authorID', $authorID );
                $stmt->bindParam( ':parentID', $parentID );
                $stmt->bindParam( ':title', $title );
                $stmt->bindParam( ':content', $content );
                $stmt->bindParam( ':url', $url );
                $stmt->bindParam( ':created', $created );
                $stmt->bindParam( ':modified', $modified );
                $result = $stmt->execute();
                OnpubDatabase::verifyExecute( $this->pdo, $result, $this->enableTransactions, $stmt->errorInfo() );

                $IDs[] = $this->pdo->lastInsertId();
                $comment->ID = $this->pdo->lastInsertId();
            }
        }

        if ( $this->enableTransactions ) {
            $status = $this->pdo->commit();
            OnpubDatabase::verifyTransaction( $this->pdo, $status );
        }

        if ( $isArray ) {
            return $IDs;
        }
        else {
            return end( $IDs );
        }
    }

    /**
     * Get multiple comments from the database.
     *
     * @param OnpubQueryOptions $queryOptions Database query options.
     * @return array An array of {@link OnpubComment} objects.
     */
    public function select( OnpubQueryOptions $queryOptions = null, $articleID = null, $authorID = null )
    {
        if ( $queryOptions === null ) $queryOptions = new OnpubQueryOptions();

        $query = $this->selectQuery( $queryOptions, $articleID, $authorID );
        $result = $this->pdo->query( $query );
        OnpubDatabase::verifyQuery( $this->pdo, $result, false );
        $rows = $result->fetchAll( PDO::FETCH_ASSOC );

        $comments = array();

        if ( $rows ) {
            foreach ( $rows as $row ) {
                $comment = new OnpubComment();

                if ( $queryOptions->includeContent ) {
                    $content = stripslashes( $row["content"] );
                }
                else {
                    $content = "";
                }

                $comment->ID = $row["ID"];
                $comment->imageID = $row["imageID"];
                $comment->articleID = $row["articleID"];
                $comment->authorID = $row["authorID"];
                $comment->parentID = $row["parentID"];
                $comment->title = stripslashes( $row["title"] );
                $comment->content = $content;
                $comment->url = stripslashes( $row["url"] );
                $comment->setCreated( new DateTime( $row["created"] ) );
                $comment->setModified( new DateTime( $row["modified"] ) );

                $comments[] = $comment;
            }
        }

        $result->closeCursor();
        return $comments;
    }

    private function selectQuery( OnpubQueryOptions $queryOptions = null, $articleID = null, $authorID = null )
    {
        if ( $queryOptions === null ) $queryOptions = new OnpubQueryOptions();

        $commentColumns = "comments.ID, comments.imageID, comments.articleID, comments.authorID, comments.parentID, comments.title, comments.url, comments.created, comments.modified";
        $where = "";
        $orderBy = "";
        $limit = "";

        if ( $queryOptions->includeContent ) {
            $commentColumns = "comments.ID, comments.imageID, comments.articleID, comments.authorID, comments.parentID, comments.title, comments.content, comments.url, comments.created, comments.modified";
        }

        if ( $queryOptions->dateLimit ) {
            $where = "WHERE comments.created <= " . $this->pdo->quote( $queryOptions->dateLimit->format( DATE_ISO8601 ) );

            if ( $articleID && !$authorID ) {
                $where .= " AND comments.articleID = $articleID";
            }

            if ( $authorID && !$articleID ) {
                $where .= " AND comments.authorID = $authorID";
            }
        }
        else {
            if ( $articleID && !$authorID ) {
                $where = "WHERE comments.articleID = $articleID";
            }

            if ( $authorID && !$articleID ) {
                $where = "WHERE comments.authorID = $authorID";
            }
        }

        if ( $queryOptions->orderBy ) {
            $orderBy = "ORDER BY comments." . $queryOptions->orderBy;

            if ( $queryOptions->order ) {
                $orderBy .= " " . $queryOptions->order;
            }
        }

        if ( $queryOptions->getPage() && $queryOptions->rowLimit && $queryOptions->rowLimit > 0 ) {
            $limit = "LIMIT " . (($queryOptions->getPage() - 1) * $queryOptions->rowLimit) . "," . $queryOptions->rowLimit;
        }
        elseif ( $queryOptions->rowLimit && $queryOptions->rowLimit > 0 ) {
            $limit = "LIMIT 0," . $queryOptions->rowLimit;
        }

        return "SELECT $commentColumns FROM OnpubComments AS comments $where $orderBy $limit";
    }

    /**
     * Update a comment already in the database.
     *
     * If you set the comment's sectionIDs to NULL, it will be unmapped from
     * any sections it was previously mapped to.
     *
     * @param OnpubComment $comment The comment to be updated.
     * @param bool $overwriteAAMaps False by default. If set to true, the
     * comment-author maps for this comment will be deleted and recreated, if
     * applicable.
     * @return int 1 if the comment was updated. 0 if the comment does not exist in the database.
     */
    public function update( OnpubComment $comment, $overwriteAAMaps = false )
    {
        $oaamaps = new OnpubAAMaps( $this->pdo, false );
        $oauthors = new OnpubAuthors( $this->pdo, false );
        $osamaps = new OnpubSAMaps( $this->pdo, false );
        $oimages = new OnpubImages( $this->pdo, false );
        $now = new DateTime();

        if ( $this->enableTransactions ) {
            $status = $this->pdo->beginTransaction();
            OnpubDatabase::verifyTransaction( $this->pdo, $status );
        }

        $stmt = $this->pdo->prepare( "UPDATE OnpubComments SET imageID = :imageID, title = :title, content = :content, url = :url, created = :created, modified = :modified WHERE ID = :ID" );
        OnpubDatabase::verifyPrepare( $this->pdo, $stmt, $this->enableTransactions );

        if ( $comment->image ) {
            try {
                $imageID = $oimages->insert( $comment->image );
                $comment->imageID = $imageID;
            }
            catch ( PDOException $e ) {
                if ( $this->enableTransactions ) $this->pdo->rollBack();
                throw $e;
            }
        }

        $ID = $comment->ID;
        $imageID = $comment->imageID;
        $title = OnpubDatabase::utf8Decode( trim( $comment->title ) );
        $content = OnpubDatabase::utf8Decode( trim( $comment->content ) );
        $url = OnpubDatabase::utf8Decode( trim( $comment->url ) );
        $created = $comment->getCreated()->format( DATE_ISO8601 );
        $modified = $now->format( DATE_ISO8601 );

        $stmt->bindParam( ':ID', $ID );
        $stmt->bindParam( ':imageID', $imageID );
        $stmt->bindParam( ':title', $title );
        $stmt->bindParam( ':content', $content );
        $stmt->bindParam( ':url', $url );
        $stmt->bindParam( ':created', $created );
        $stmt->bindParam( ':modified', $modified );
        $result = $stmt->execute();
        OnpubDatabase::verifyExecute( $this->pdo, $result, $this->enableTransactions, $stmt->errorInfo() );

        if ( $overwriteAAMaps ) {
            try {
                $oaamaps->delete( $comment->ID, null );
            }
            catch ( PDOException $e ) {
                if ( $this->enableTransactions ) $this->pdo->rollBack();
                throw $e;
            }
        }

        $author = $comment->author;

        foreach ( $author as $author ) {
            if ( $author->ID ) {
                try {
                    $oauthors->update( $author );
                }
                catch ( PDOException $e ) {
                    if ( $this->enableTransactions ) $this->pdo->rollBack();
                    throw $e;
                }
            }
            else {
                try {
                    $oauthors->insert( $author );
                }
                catch ( PDOException $e ) {
                    if ( $this->enableTransactions ) $this->pdo->rollBack();
                    throw $e;
                }
            }

            try {
                $aamap = new OnpubAAMap();
                $aamap->setCommentID( $comment->ID );
                $aamap->authorID = $author->ID;
                $oaamaps->insert( $aamap );
            }
            catch ( PDOException $e ) {
                if ( $this->enableTransactions ) $this->pdo->rollBack();
                throw $e;
            }
        }

        $sectionIDs = $comment->sectionIDs;

        if ( $sectionIDs === null ) {
            try {
                $samaps = $osamaps->delete( null, $comment->ID );
            }
            catch ( PDOException $e ) {
                if ( $this->enableTransactions ) $this->pdo->rollBack();
                throw $e;
            }
        }
        elseif ( sizeof( $sectionIDs ) ) {
            $queryOptions = new OnpubQueryOptions();
            $queryOptions->orderBy = "ID";
            $queryOptions->order = "ASC";

            try {
                $samaps = $osamaps->select( $queryOptions, null, $comment->ID );
            }
            catch ( PDOException $e ) {
                if ( $this->enableTransactions ) $this->pdo->rollBack();
                throw $e;
            }

            // Unmap sections not included in $sectionIDs.
            foreach ( $samaps as $samap ) {
                if ( !in_array( $samap->sectionID, $sectionIDs ) ) {
                    try {
                        $osamaps->delete( $samap->sectionID, $comment->ID );
                    }
                    catch ( PDOException $e ) {
                        if ( $this->enableTransactions ) $this->pdo->rollBack();
                        throw $e;
                    }
                }
            }

            foreach ( $sectionIDs as $sectionID ) {
                $samap = new OnpubSAMap();

                $samap->sectionID = $sectionID;
                $samap->setCommentID( $comment->ID );
                $samap->setCreated( $comment->getCreated() );
                $samap->setModified( $comment->getModified() );

                try {
                    $samapID = $osamaps->getID( $samap );
                }
                catch ( PDOException $e ) {
                    if ( $this->enableTransactions ) $this->pdo->rollBack();
                    throw $e;
                }

                if ( $samapID ) {
                    $samap->ID = $samapID;

                    try {
                        $osamaps->update( $samap );
                    }
                    catch ( PDOException $e ) {
                        if ( $this->enableTransactions ) $this->pdo->rollBack();
                        throw $e;
                    }
                }
                else {
                    try {
                        $osamaps->insert( $samap );
                    }
                    catch ( PDOException $e ) {
                        if ( $this->enableTransactions ) $this->pdo->rollBack();
                        throw $e;
                    }
                }
            }
        }

        if ( $this->enableTransactions ) {
            $status = $this->pdo->commit();
            OnpubDatabase::verifyTransaction( $this->pdo, $status );
        }

        return $stmt->rowCount();
    }
}

?>
